# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from activitysmith_openapi.models.metric_format import MetricFormat
from activitysmith_openapi.models.metric_unit_spacing import MetricUnitSpacing
from activitysmith_openapi.models.widget_metric_latest_value import WidgetMetricLatestValue
from typing import Optional, Set
from typing_extensions import Self

class WidgetMetric(BaseModel):
    """
    A metric configured for ActivitySmith widgets.
    """ # noqa: E501
    public_id: StrictStr
    key: Annotated[str, Field(min_length=1, strict=True, max_length=64)]
    label: Annotated[str, Field(min_length=1, strict=True, max_length=80)]
    currency_code: Optional[Annotated[str, Field(min_length=3, strict=True, max_length=3)]] = Field(description="Present when format is currency.")
    unit: Optional[Annotated[str, Field(strict=True, max_length=16)]] = Field(description="Present when format is unit.")
    unit_spacing: MetricUnitSpacing
    format: MetricFormat
    latest_value: Optional[WidgetMetricLatestValue]
    latest_value_at: Optional[datetime]
    created_at: datetime
    updated_at: datetime
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["public_id", "key", "label", "currency_code", "unit", "unit_spacing", "format", "latest_value", "latest_value_at", "created_at", "updated_at"]

    @field_validator('key')
    def key_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9][a-z0-9_.-]{0,63}$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9][a-z0-9_.-]{0,63}$/")
        return value

    @field_validator('currency_code')
    def currency_code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[A-Z]{3}$", value):
            raise ValueError(r"must validate the regular expression /^[A-Z]{3}$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of WidgetMetric from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of latest_value
        if self.latest_value:
            _dict['latest_value'] = self.latest_value.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['currency_code'] = None

        # set to None if unit (nullable) is None
        # and model_fields_set contains the field
        if self.unit is None and "unit" in self.model_fields_set:
            _dict['unit'] = None

        # set to None if latest_value (nullable) is None
        # and model_fields_set contains the field
        if self.latest_value is None and "latest_value" in self.model_fields_set:
            _dict['latest_value'] = None

        # set to None if latest_value_at (nullable) is None
        # and model_fields_set contains the field
        if self.latest_value_at is None and "latest_value_at" in self.model_fields_set:
            _dict['latest_value_at'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of WidgetMetric from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "public_id": obj.get("public_id"),
            "key": obj.get("key"),
            "label": obj.get("label"),
            "currency_code": obj.get("currency_code"),
            "unit": obj.get("unit"),
            "unit_spacing": obj.get("unit_spacing"),
            "format": obj.get("format"),
            "latest_value": WidgetMetricLatestValue.from_dict(obj["latest_value"]) if obj.get("latest_value") is not None else None,
            "latest_value_at": obj.get("latest_value_at"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


