# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from activitysmith_openapi.models.metric_value_update_request import MetricValueUpdateRequest

class TestMetricValueUpdateRequest(unittest.TestCase):
    """MetricValueUpdateRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> MetricValueUpdateRequest:
        """Test MetricValueUpdateRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `MetricValueUpdateRequest`
        """
        model = MetricValueUpdateRequest()
        if include_optional:
            return MetricValueUpdateRequest(
                value = None,
                timestamp = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return MetricValueUpdateRequest(
                value = None,
        )
        """

    def testMetricValueUpdateRequest(self):
        """Test MetricValueUpdateRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
