# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from activitysmith_openapi.models.widget_metric import WidgetMetric

class TestWidgetMetric(unittest.TestCase):
    """WidgetMetric unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> WidgetMetric:
        """Test WidgetMetric
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `WidgetMetric`
        """
        model = WidgetMetric()
        if include_optional:
            return WidgetMetric(
                public_id = '',
                key = 'w6otgfww_r74cm.fyln9uyrj1gzi.i4v8m.oy3c3lv1-42d1z89qs7dlt2ggwxdm0',
                label = '0',
                currency_code = 'AEI012',
                unit = '',
                unit_spacing = 'none',
                format = 'number',
                latest_value = 1.337,
                latest_value_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return WidgetMetric(
                public_id = '',
                key = 'w6otgfww_r74cm.fyln9uyrj1gzi.i4v8m.oy3c3lv1-42d1z89qs7dlt2ggwxdm0',
                label = '0',
                currency_code = 'AEI012',
                unit = '',
                unit_spacing = 'none',
                format = 'number',
                latest_value = 1.337,
                latest_value_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
        )
        """

    def testWidgetMetric(self):
        """Test WidgetMetric"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
