# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from activitysmith_openapi.models.metric_unit_spacing import MetricUnitSpacing

class TestMetricUnitSpacing(unittest.TestCase):
    """MetricUnitSpacing unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testMetricUnitSpacing(self):
        """Test MetricUnitSpacing"""
        # inst = MetricUnitSpacing()

if __name__ == '__main__':
    unittest.main()
